@@ -24,9 +24,6 @@
 
 #include <Lum/OS/Driver.h>
 
-#include <iostream>
-static GameArea::Prefs *prefs=new GameArea::Prefs();
-
 GameArea::GameArea()
  : themeIndex(0),
    xOff(0),
@@ -38,9 +35,8 @@ GameArea::GameArea()
    frameShineColor("grey95",Lum::OS::display->GetColor(Lum::OS::Display::blackColor)),
    frameShadowColor("grey20",Lum::OS::display->GetColor(Lum::OS::Display::blackColor))
 {
-  SetPrefs(::prefs);
-
   SetCanFocus(true);
+  RequestFocus();
 
   finished->Set(false);
 }
@@ -81,99 +77,18 @@ size_t GameArea::GetSokobanYPos() const
   return sy;
 }
 
-bool GameArea::IsAreaCompletelyVisible() const
-{
-  return sokoban.GetWidth()<=width/GetBoxSize() && sokoban.GetHeight()<=height/GetBoxSize();
-}
-
-bool GameArea::CanScrollLeft() const
-{
-  return xOff>0;
-}
-
-bool GameArea::CanScrollUp() const
-{
-  return yOff>0;
-}
-
-bool GameArea::CanScrollRight() const
-{
-  return xOff+(width/GetBoxSize()-2)<sokoban.GetWidth();
-}
-
-bool GameArea::CanScrollDown() const
-{
-  return yOff+(height/GetBoxSize()-2)<sokoban.GetHeight();
-}
-
 void GameArea::MakeSokobanVisible()
 {
-  // If the area is completely visible, there is no need to move the
-  // area offset to make the sokoban visible!
-  if (IsAreaCompletelyVisible()) {
-    return;
-  }
-
   size_t sx;
   size_t sy;
 
   sokoban.GetSokoban(sx,sy);
 
-  if (sx<xOff || sx>xOff+(width/GetBoxSize()-2)-1) {
-    if (sx<(width/GetBoxSize()-2)/2) {
-      xOff=0;
-    }
-    else if (sx>sokoban.GetWidth()-(width/GetBoxSize()-2)/2) {
-      xOff=sokoban.GetWidth()-(width/GetBoxSize()-2);
-    }
-    else {
-      xOff=sx-(width/GetBoxSize()-2)/2;
-    }
-  }
+  hAdjustment->MakeVisible(sx*GetBoxSize()+1);
+  vAdjustment->MakeVisible(sy*GetBoxSize()+1);
 
-  if (sy<yOff || sy>yOff+(height/GetBoxSize()-2)-1) {
-    if (sy<(height/GetBoxSize()-2)/2) {
-      yOff=0;
-    }
-    else if (sy>sokoban.GetHeight()-(height/GetBoxSize()-2)/2) {
-      yOff=sokoban.GetHeight()-(height/GetBoxSize()-2);
-    }
-    else {
-      yOff=sy-(height/GetBoxSize()-2)/2;
-    }
-  }
-}
-
-void GameArea::ScrollLeft()
-{
-  assert(xOff>0);
-
-  xOff--;
-  Redraw();
-}
-
-void GameArea::ScrollUp()
-{
-  assert(yOff>0);
-
-  yOff--;
-  Redraw();
-}
-
-void GameArea::ScrollRight()
-{
-  //assert();
-
-  xOff++;
-  Redraw();
-}
-
-void GameArea::ScrollDown()
-{
-  //assert();
-
-  yOff++;
-  Redraw();
+  hAdjustment->MakeVisible((sx+1)*GetBoxSize()+1);
+  vAdjustment->MakeVisible((sy+1)*GetBoxSize()+1);
 }
 
 void GameArea::MoveLeft()
@@ -365,6 +280,9 @@ void GameArea::RebuildGameBackground()
       }
     }
   }
+
+  hAdjustment->SetDimension(width,gameBackground->GetWidth());
+  vAdjustment->SetDimension(height,gameBackground->GetHeight());
 }
 
 void GameArea::RebuildGame()
@@ -439,6 +357,11 @@ void GameArea::SetSokoban(const Sokoban& sokoban)
   finished->Set(false);
 }
 
+bool GameArea::HandlesFocus() const
+{
+  return RequestedFocus() && visible;
+}
+
 void GameArea::CalcSize()
 {
   if (Lum::OS::display->GetSize()<Lum::OS::Display::sizeNormal) {
@@ -453,14 +376,24 @@ void GameArea::CalcSize()
   minWidth=10*32;
   minHeight=10*32;
 
-  Object::CalcSize();
+  Scrollable::CalcSize();
+}
+
+void GameArea::Layout()
+{
+  if (gameBackground!=NULL) {
+    hAdjustment->SetDimension(width,gameBackground->GetWidth());
+    vAdjustment->SetDimension(height,gameBackground->GetHeight());
+  }
+
+  Scrollable::Layout();
 }
 
 void GameArea::Draw(int x, int y, size_t w, size_t h)
 {
-  Object::Draw(x,y,w,h);
+  Scrollable::Draw(x,y,w,h);
 
-  if (!Intersect(x,y,w,h)) {
+  if (!OIntersect(x,y,w,h)) {
     return;
   }
 
@@ -468,72 +401,16 @@ void GameArea::Draw(int x, int y, size_t w, size_t h)
 
   DrawBackground(this->x,this->y,this->width,this->height);
 
-  if (sokoban.GetWidth()==0 || sokoban.GetHeight()==0) {
+  if (game==NULL) {
     return;
   }
 
   Lum::OS::DrawInfo *draw=GetDrawInfo();
 
-  if (IsAreaCompletelyVisible()) {
-    draw->CopyFromBitmap(game,
-                         0,0,sokoban.GetWidth()*GetBoxSize(),sokoban.GetHeight()*GetBoxSize(),
-                         this->x,this->y);
-  }
-  else {
-    draw->CopyFromBitmap(game,
-                         xOff*GetBoxSize(),yOff*GetBoxSize(),
-                         width-2*GetBoxSize(),
-                         height-2*GetBoxSize(),
-                         this->x+GetBoxSize(),this->y+GetBoxSize());
-
-    // Up
-    if (CanScrollUp()) {
-      if (GetImage(upImage)!=NULL) {
-        GetImage(upImage)->Draw(draw,
-                                this->x+(width-GetBoxSize())/2,
-                                this->y);
-      }
-      else {
-        // TODO
-      }
-    }
-
-    // Left
-    if (CanScrollLeft()) {
-      if (GetImage(leftImage)!=NULL) {
-        GetImage(leftImage)->Draw(draw,
-                                  this->x,
-                                  this->y+(height-GetBoxSize())/2);
-      }
-      else {
-        // TODO
-      }
-    }
-
-    // Down
-    if (CanScrollDown()) {
-      if (GetImage(downImage)!=NULL) {
-        GetImage(downImage)->Draw(draw,
-                                  this->x+(width-GetBoxSize())/2,
-                                  this->y+height-GetBoxSize());
-      }
-      else {
-        // TODO
-      }
-    }
-
-    // Right
-    if (CanScrollRight()) {
-      if (GetImage(rightImage)!=NULL) {
-        GetImage(rightImage)->Draw(draw,
-                                   this->x+width-GetBoxSize(),
-                                   this->y+(height-GetBoxSize())/2);
-      }
-      else {
-        // TODO
-      }
-    }
-  }
+  draw->CopyFromBitmap(game,
+                       hAdjustment->GetTop()-1,vAdjustment->GetTop()-1,
+                       width,height,
+                       this->x,this->y);
 }
 
 bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
@@ -542,31 +419,8 @@ bool GameArea::HandleMouseEvent(const Lum::OS::MouseEvent& event)
     size_t x,y;
     size_t sx,sy;
 
-    if (!IsAreaCompletelyVisible()) {
-      if ((size_t)event.x<this->x+GetBoxSize() && CanScrollLeft()) {
-        ScrollLeft();
-        return true;
-      }
-      if ((size_t)event.x>=this->x+width-GetBoxSize() && CanScrollRight()) {
-        ScrollRight();
-        return true;
-      }
-      if ((size_t)event.y<this->y+GetBoxSize() && CanScrollUp()) {
-        ScrollUp();
-        return true;
-      }
-      if ((size_t)event.y>=this->y+height-GetBoxSize() && CanScrollDown()) {
-        ScrollDown();
-        return true;
-      }
-
-      x=(event.x-this->x)/GetBoxSize()-1+xOff;
-      y=(event.y-this->y)/GetBoxSize()-1+yOff;
-    }
-    else {
-      x=(event.x-this->x)/GetBoxSize();
-      y=(event.y-this->y)/GetBoxSize();
-    }
+    x=(event.x-this->x+hAdjustment->GetTop()-1)/GetBoxSize();
+    y=(event.y-this->y+vAdjustment->GetTop()-1)/GetBoxSize();
 
     sokoban.GetSokoban(sx,sy);
 
@@ -673,3 +527,12 @@ void GameArea::Undo()
   finished->Set(sokoban.IsFinished());
 }
 
+void GameArea::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
+{
+  if (model==hAdjustment->GetTopModel() || model==vAdjustment->GetTopModel()) {
+    Redraw();
+  }
+
+  Lum::Scrollable::Resync(model,msg);
+}
+
