@@ -32,6 +32,7 @@
 #include <Lum/Table.h>
 #include <Lum/Text.h>
 #include <Lum/TextValue.h>
+#include <Lum/View.h>
 #include <Lum/WindowGroup.h>
 
 #include <Lum/Dlg/About.h>
@@ -116,7 +117,6 @@ public:
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->RequestFocus();
     table->SetShowHeader(true);
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
@@ -132,7 +132,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(okAction);
     button->SetType(Lum::Button::typeCommit);
     button->SetText(L"_Ok");
@@ -140,7 +139,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(GetClosedAction());
     button->SetType(Lum::Button::typeCancel);
     button->SetText(L"_Cancel");
@@ -233,7 +231,6 @@ public:
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->RequestFocus();
     table->SetModel(list);
     table->SetSelection(selection);
     table->SetDoubleClickAction(okAction);
@@ -247,7 +244,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(okAction);
     button->SetType(Lum::Button::typeCommit);
     button->SetText(L"_Ok");
@@ -255,7 +251,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(GetClosedAction());
     button->SetType(Lum::Button::typeCancel);
     button->SetText(L"_Cancel");
@@ -362,7 +357,6 @@ public:
 
     table=new Lum::Table();
     table->SetFlex(true,true);
-    table->RequestFocus();
     table->SetShowHeader(true);
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
@@ -378,7 +372,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(okAction);
     button->SetType(Lum::Button::typeCommit);
     button->SetText(L"_Ok");
@@ -386,7 +379,6 @@ public:
 
     button=new Lum::Button();
     button->SetFlex(true,true);
-    button->RequestFocus();
     button->SetModel(GetClosedAction());
     button->SetType(Lum::Button::typeCancel);
     button->SetText(L"_Cancel");
@@ -494,6 +486,7 @@ public:
     Lum::Panel       *horiz2;
     Lum::Panel       *vert;
     Lum::TextValue   *textValue;
+    Lum::View        *view;
     Lum::WindowGroup *wGroup;
 
     wGroup=new Lum::WindowGroup();
@@ -505,9 +498,13 @@ public:
     vert=new Lum::VPanel();
     vert->SetFlex(true,true);
 
+    view=new Lum::View();
+    view->SetFlex(true,true);
+
     gameArea->SetFlex(true,true);
-    gameArea->RequestFocus();
-    vert->Add(gameArea);
+    view->SetObject(gameArea);
+
+    vert->Add(view);
 
     horiz->Add(vert);
 
@@ -687,14 +684,12 @@ public:
                               L"the program, or have deleted the history of solved levels!");
       }
 
+      ChooseTheme();
+      ChooseCollection();
+
       gameArea->EvaluateThemeName();
       themeName->Set(::themeName);
-      if (collection!=NULL) {
-        SetCollection(collection);
-      }
-      else {
-        SetCollection(Collection::GetCollection(0));
-      }
+      SetCollection(::collection);
     }
     else if (model==categoryAction && categoryAction->IsFinished()) {
       SelectCollection *dialog=new SelectCollection();
